// -*- c-basic-offset: 4 -*-
// PanoImage.cpp
//
// Pablo d'Angelo <pablo@mathematik.uni-ulm.de>
// Last change: Time-stamp: <21-Apr-2003 22:46:06 pablo@island.wh-wurm.uni-ulm.de>
//
//

#include <stdio.h>
#include <math.h>
//#include <setjmp.h>

#include <stdexcept>

extern "C" {
#include <jpeglib.h>
}

#include "PanoImage.h"
#include "Panorama.h"
#include "utils.h"



using namespace PT;
using namespace std;


#if 0
QDomElement ImagePosition::toXML(QDomDocument & doc)
{
    QDomElement root = doc.createElement("position");

    root.setAttribute("yaw", yaw);
    root.setAttribute("pitch", pitch);
    root.setAttribute("roll", roll);

    return root;
}


void ImagePosition::setFromXML(const QDomNode & node)
{
    DEBUG_DEBUG("ImagePosition::setFromXML");
    Q_ASSERT(node.nodeName() == "position");
    QDomNamedNodeMap attrs = node.attributes();
    yaw = getAttrib(attrs, "yaw").toDouble();
    roll = getAttrib(attrs, "roll").toDouble();
    pitch = getAttrib(attrs, "pitch").toDouble();
}


QDomElement ImageOptions::toXML(QDomDocument & doc)
{
    QDomElement root = doc.createElement("options");
    root.setAttribute("feather_width", featherWidth);
    root.setAttribute("ignore_frame_width", ignoreFrameWidth);
    root.setAttribute("morph", morph);
    root.setAttribute("optimize_yaw", optimizeYaw);
    root.setAttribute("optimize_roll", optimizeRoll);
    root.setAttribute("optimize_pitch", optimizePitch);
    root.setAttribute("optimize_HFOV", optimizeFOV);
    root.setAttribute("optimize_a", optimizeA);
    root.setAttribute("optimize_b", optimizeB);
    root.setAttribute("optimize_c", optimizeC);
    root.setAttribute("optimize_d", optimizeD);
    root.setAttribute("optimize_e", optimizeE);

    return root;
}


void ImageOptions::setFromXML(const QDomNode & node)
{
    DEBUG_DEBUG("ImageOptions::setFromXML");
    Q_ASSERT(node.nodeName() == "options");
    QDomNamedNodeMap attrs = node.attributes();
    featherWidth = getAttrib(attrs, "feather_width").toUInt();
    ignoreFrameWidth = getAttrib(attrs, "ignore_frame_width").toUInt();
    morph = getAttrib(attrs, "morph").toUInt() != 0;
}

#endif

PanoImage::PanoImage(const std::string &filename)
    : filename(filename)
{
    DEBUG_DEBUG("PanoImage ctor for " << filename);
    init();
    readImageInformation();
}

void PanoImage::init()
{
    height = 0;
    width = 0;
}


#if 0
PanoImage::PanoImage(QDomNode & node)
{
    setFromXML(node);
}
#endif


PanoImage::~PanoImage()
{
    DEBUG_DEBUG("PanoImage dtor, image= " << filename);
}

#if 0
void PanoImage::setFromXML(QDomNode & elem)
{
    assert(elem.nodeName() == "image");
    QDomNamedNodeMap attrs = elem.attributes();
    filename = getAttrib(attrs, "file");
    isLandscape = getAttrib(attrs, "landscape").toUInt() != 0;
    height = getAttrib(attrs, "height").toUInt();
    width = getAttrib(attrs, "width").toUInt();

    qDebug("PanoImage::setFromXML, file %s", filename.ascii());

    options.setFromXML(elem.namedItem("position"));
    lens.setFromXML(elem.namedItem("lens"));
    options.setFromXML(elem.namedItem("options"));
}


QDomElement PanoImage::toXML(QDomDocument & doc)
{
    QDomElement root = doc.createElement("image");
    root.setAttribute("file", filename);
    root.setAttribute("landscape", isLandscape);
    root.setAttribute("height",height);
    root.setAttribute("width",width);
    root.appendChild(lens.toXML(doc));
    root.appendChild(position.toXML(doc));
    root.appendChild(options.toXML(doc));
    return root;
}
#endif

bool PanoImage::readImageInformation()
{
  bool ok;
  std::string::size_type idx = filename.rfind('.');
  if (idx == std::string::npos) {
      DEBUG_DEBUG("could not find extension in filename");
      return false;
  }
  std::string ext = filename.substr( idx+1 );

  if (ext == "jpg" || ext == "JPG") {
    ok = readJPEGInfo();
  } else if (ext == "tiff" || ext == "tif" || ext == "TIFF" || ext == "TIF") {
    ok = readTIFFInfo();
  } else if (ext == "png" || ext == "PNG") {
    ok = readPNGInfo();
  } else {
      DEBUG_NOTICE("Unknown Image format: " << filename);
    return false;
  }
  if (!ok) {
      DEBUG_NOTICE("error while reading file information for " << filename);
    return false;
  }
//  isLandscape = (width > height);
  return true;
}

bool PanoImage::readJPEGInfo()
{
  // read normal jpeg header for image dimensions

  /* This struct contains the JPEG decompression parameters and pointers to
   * working space (which is allocated as needed by the JPEG library).
   */
  struct jpeg_decompress_struct cinfo;
  /* We use our private extension JPEG error handler.
   * Note that this struct must live as long as the main JPEG parameter
   * struct, to avoid dangling-pointer problems.
   */
  struct jpeg_error_mgr jerr;
  /* More stuff */
  FILE * infile;                /* source file */

  /* In this example we want to open the input file before doing anything else,
   * so that the setjmp() error recovery below can assume the file is open.
   * VERY IMPORTANT: use "b" option to fopen() if you are on a machine that
   * requires it in order to read binary files.
   */

  if ((infile = fopen(filename.c_str(), "rb")) == NULL) {
      DEBUG_NOTICE("can't open " << filename.c_str()<< ": " << strerror(errno));
      return false;
  }

  /* Step 1: allocate and initialize JPEG decompression object */

  /* We set up the normal JPEG error routines */
  cinfo.err = jpeg_std_error(&jerr);
  /* Now we can initialize the JPEG decompression object. */
  jpeg_create_decompress(&cinfo);

  /* Step 2: specify data source (eg, a file) */

  jpeg_stdio_src(&cinfo, infile);

  /* Step 3: read file parameters with jpeg_read_header() */

  (void) jpeg_read_header(&cinfo, TRUE);

  jpeg_destroy_decompress(&cinfo);
  fclose (infile);

  width = cinfo.image_width;
  height = cinfo.image_height;

  return true;
}

bool PanoImage::readTIFFInfo()
{
  DEBUG_ERROR("readTIFFInfo not implemented");
  return false;
}

bool PanoImage::readPNGInfo()
{
  DEBUG_ERROR("readPNGInfo not implemented");
  return false;
}

#if 0
Magick::Image PanoImage::getImage()
{
    if (pixmap.isNull()) {
        DEBUG_DEBUG("Loading image data from %s",filename.ascii());
        if (!pixmap.load(filename)) {
            qWarning("Could not load image file: %s", filename.ascii());
        }
    }
    return pixmap;
}
#endif
